<?php
/**
 * Copyright © FiloBlu S.p.A. 2019. All rights reserved.
 */

namespace FiloBlu\Rma\Plugin\Helper;
use FiloBlu\Rma\Helper\ChangeSizeOrder;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Sales\Model\Order\ItemRepository;
use Magento\Catalog\Model\ProductRepository;
use Magento\ConfigurableProduct\Api\LinkManagementInterface;
use FiloBlu\Rma\Model\ChangeSize;
use Magento\Sales\Model\OrderRepository;
use FiloBlu\Rma\Model\Payment\ChangeSizePay;



class Data
{
    /**
     * @var ScopeConfigInterface 
     */
    protected $scopeConfig;
    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var LoggerInterface
     */
    protected $logger;


    const XML_PATH_VALIDITY_DAYS ='sales/rma_validity_settings/validity_days';

    const ALLOW_CHANGESIZEORDER_TO_USE_CHANGESIZE = 'filoblu_disablerma_section/rma_group/rma_change_sizeorder_can_use_changesize';
    //const XML_PATH_DAYS_AUTO_APPROVAL ='sales/rma_validity_settings/automatic_approval';

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var ChangeSize
     */
    protected $changeSize;


    /**
     * @var ItemRepository
     */
    protected $itemRepository;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LinkManagementInterface
     */
    protected $linkManagment;

    /**
     * @var ChangeSizePay
     */
    protected $changeSizePay;


    /**
     * Data constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param OrderRepositoryInterface $orderRepository
     * @param LoggerInterface $logger
     * @param RequestInterface $request
     * @param ItemRepository $itemRepository
     * @param ProductRepository $productRepository
     * @param LinkManagementInterface $linkManagement
     * @param ChangeSize $changeSize
     * @param ChangeSizePay $changeSizePay
     */
    public function __construct(ScopeConfigInterface $scopeConfig,
                                OrderRepositoryInterface $orderRepository,
                                LoggerInterface $logger,
                                RequestInterface $request,
                                ItemRepository $itemRepository,
                                ProductRepository $productRepository,
                                LinkManagementInterface $linkManagement,
                                ChangeSize $changeSize,
                                ChangeSizePay $changeSizePay){
        
        $this->scopeConfig = $scopeConfig;
        $this->orderRepository = $orderRepository;
        $this->logger = $logger;
        $this->request = $request;
        $this->itemRepository =$itemRepository;
        $this->productRepository = $productRepository;
        $this->linkManagment = $linkManagement;
        $this->changeSize = $changeSize;
        $this->changeSizePay = $changeSizePay;
    }


    /**
     * @param $order
     * @return bool
     */
    public function isValid($order)
    {
        //error_log(__METHOD__);

        $validity_days = $this->scopeConfig->getValue(
            self::XML_PATH_VALIDITY_DAYS,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

       try{
           if(!is_object($order)){
               $order = $this->orderRepository->get($order);
           }

           $orderDate = $order->getCreatedAt(); //2019-11-06 13:17:01
           $dateTimeCreated = new \DateTime($orderDate);
           $dateTimeCreated->setTime(0,0,0);
           $dateTimeValidityEnds = $dateTimeCreated;
           $dateTimeValidityEnds->modify('+'.$validity_days.' day');
           $today = new \DateTime();
           $today->setTime(0,0,0);
           if($today <= $dateTimeValidityEnds){
               return true;
           }else{
               return false;
           }


       }catch (\Throwable $throwable){
           $this->logger->critical($throwable->getMessage());
           return false;
       }catch (\Exception $e){
           $this->logger->critical($e->getMessage());
           return false;
       }



    }


    /** Magento 2.1 has problem with after plugin with arguments
     * @param \Magento\Rma\Helper\Data $subject
     * @param callable $proceed
     * @param $order
     * @param bool $forceCreate
     * @return bool
     */
    public function aroundCanCreateRma(\Magento\Rma\Helper\Data $subject , callable $proceed, $order, $forceCreate = false)
    {
        if($forceCreate == true){
            return $forceCreate;
        }
        if(!$this->checkAvailability()){
            return false;
        }
        $result = $proceed($order,$forceCreate);
        if(!$result){
            return $result;
        }else{
            $result = $this->isValid($order);
            return $result;
        }
    }


    /**
     * @return bool
     */
    public function checkAvailability(){
        
        try{
            $items = $this->request->getParam('items') ?? [];
            if(count($items) == 0){
                return true;
            }
            foreach ($items as $item) {
                if(isset($item['change_size'])){

                    $itemId = $item['change_size'];
                    $simpleProduct = $this->productRepository->getById($itemId);
                    if(!$this->changeSize->isSaleable($simpleProduct)){
                        $this->logger->critical("ChangeSize Rma : product with sku ".$simpleProduct->getSku()." is not salable!");
                        return false;
                    }else{
                        if (isset($item['qty_requested']) && $item['qty_requested'] > 0 && $this->changeSize->getProductQtyById($simpleProduct->getId()) >= $item['qty_requested']) {
                            if($this->allowChangeSizeOrderRma()){
                                $params = $this->request->getParams();
                                if(isset($params['order_id'])){
                                    $orderId =$params['order_id'];
                                }
                                $this->logger->info("ChangeSize Rma : changeSize allowed on ChangeSizeOrder wiht id $orderId by BE configuration.");
                                return true;
                            }
                            if(!$this->isChangeSizeOrder()){
                                return true;
                            }else{
                                $this->logger->info("ChangeSize Rma: can't create changeSize rma because of this is a Change Size Order");
                                return false;
                            }
                            return true;
                        }else{
                            return false;
                        }
                    }

                }else{
                    return true;
                }
            }
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return false;
        }


    }

    /**
     * @return bool
     */
    public function isChangeSizeOrder(){
        try{
            $params = $this->request->getParams();
            if(isset($params['order_id'])){
                $orderId = $params['order_id'];
                $order = $this->orderRepository->get($orderId);
                if($order->getPayment()->getMethod() == $this->changeSizePay->getCode()){
                    return true;
                }else{
                    return false;
                }
            }
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return true;
        }

    }


    /**
     * @return bool
     */
    public function allowChangeSizeOrderRma(){
        return (bool)$this->scopeConfig->getValue(self::ALLOW_CHANGESIZEORDER_TO_USE_CHANGESIZE,\Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE) ?? false;
    }

}
